package common.caching;

import java.util.Date;
import java.util.Arrays;
import java.text.DateFormat;
import java.io.Serializable;

//import org.apache.jcs.engine.memory.MemoryCache;
//import org.apache.jcs.engine.behavior.ICache;
import org.apache.jcs.engine.behavior.ICacheElement;
import org.apache.jcs.engine.behavior.IElementAttributes;
import org.apache.jcs.engine.control.CompositeCacheManager;
import org.apache.jcs.engine.control.CompositeCache;

import common.caching.CachingException;


public class CacheElementInfo {

	public CacheElementInfo() {
	}

	private String key = null;
	private boolean eternal = false;
	private String createTime = null;
	private long maxLifeSeconds = -1;
	private long expiresInSeconds = -1;

	private int hitCountAux = 0;
	private int hitCountRam = 0;
	private int missCountExpired = 0;
	private int missCountNotFound = 0;
   private long versionNumber = 0;

   private double totalMemory;
   private double freeMemory;
   private double usedMemory;
   private double percentFree;

   private CompositeCacheManager cacheHub = CompositeCacheManager.getInstance();

//	public String getKey() {
//		return key;
//	}
//
//	public boolean isEternal() {
//		return eternal;
//	}
//
//	public String getCreateTime() {
//		return createTime;
//	}
//
//	public long getMaxLifeSeconds() {
//		return maxLifeSeconds;
//	}
//
//	public long getExpiresInSeconds() {
//		return expiresInSeconds;
//	}
//
//	public int getHitCountAux() {
//		return hitCountAux;
//	}
//
//	public int getHitCountRam() {
//		return hitCountRam;
//	}
//
//	public int getMissCountExpired() {
//		return missCountExpired;
//	}
//
//	public int getMissCountNotFound() {
//		return missCountNotFound;
//	}

	public void loadCacheStats(String cacheName) throws CachingException {

		CompositeCache cache = (CompositeCache)cacheHub.getCache(cacheName);
		Object[] keys = cache.getMemoryCache().getKeyArray();

		// Attempt to sort keys according to their natural ordering. If that
		// fails, get the key array again and continue unsorted.
		try {
			Arrays.sort(keys);
		} catch (Exception e) {
			keys = cache.getMemoryCache().getKeyArray();
		}

		ICacheElement element;
		IElementAttributes attributes;

		DateFormat format = DateFormat.getDateTimeInstance(DateFormat.FULL, DateFormat.FULL);
		long now = System.currentTimeMillis();

      try {

         for (int i = 0; i < keys.length; i++) {
            element = cache.getMemoryCache().getQuiet((Serializable) keys[i]);
            attributes = element.getElementAttributes();

            this.hitCountAux = cache.getHitCountAux();
            this.hitCountRam = cache.getHitCountRam();
            this.missCountExpired = cache.getMissCountExpired();
            this.missCountNotFound = cache.getMissCountNotFound();

            this.key = String.valueOf(keys[i]);
            this.eternal = attributes.getIsEternal();
            this.maxLifeSeconds = attributes.getMaxLifeSeconds();

            this.versionNumber = attributes.getVersion();

            this.createTime = format.format(new Date(attributes.getCreateTime()));
            this.expiresInSeconds = (now - attributes.getCreateTime()
                                     - (attributes.getMaxLifeSeconds() * 1000))/-1000;

            if (this.expiresInSeconds < 0) this.expiresInSeconds = 0;

            // Memory stats
            Runtime runtime = Runtime.getRuntime();
            this.totalMemory = runtime.totalMemory();
            this.freeMemory = runtime.freeMemory();
            this.usedMemory  = totalMemory - freeMemory;
            this.percentFree = freeMemory / totalMemory * 100.0;
         }
      } catch (Exception e) {
         throw new CachingException("Exception in loadCacheStats() : ", e);
      }
	}

	public String toString() {
		StringBuffer ts = new StringBuffer();
      ts.append("\n key : " + key);
      ts.append("\n createTime : " + createTime);
      ts.append("\n maxLifeSeconds : " + maxLifeSeconds);
      ts.append("\n expiresInSeconds : " + expiresInSeconds);

      ts.append("\n hitCountAux : " + hitCountAux);
      ts.append("\n hitCountRam : " + hitCountRam);
      ts.append("\n missCountExpired : " + missCountExpired);
      ts.append("\n missCountNotFound : " + missCountNotFound);
      ts.append("\nversionNumber : " + versionNumber);

      ts.append("\n totalMemory : " + totalMemory + " bytes");
      ts.append("\n freeMemory : " + freeMemory + " bytes");
      ts.append("\n usedMemory : " + usedMemory + " bytes");
      ts.append("\n percentFree : " + percentFree + " %");

		return ts.toString();
	}
}

// End of CacheElementInfo.java
