package common.caching;

import java.util.*;

import org.apache.jcs.JCS;
import common.logging.WebPortalLogger;

//import common.exception.CachingException;

public class CacheList {

	private static CacheList instance = null;

//	private Map table = new HashMap();
   private Map cacheMap = new HashMap();
   private Map cacheLoaderMap = new HashMap();
   private int scheduleExpiryItemCount = 0;

	// CONSTRUCTOR - SINGLETON PATTERN //
	private CacheList() {
	}

	public static CacheList getInstance() {
		if (instance == null) {
			instance = new CacheList();
		}
		return instance;
	}

//   public Object getCache(String regionName) {
//		CacheTable cacheTable = (CacheTable)table.get(regionName);
//      if (cacheTable == null || cacheTable.getCache() == null) return null;
//      WebPortalLogger.debug("JCS Cache for " + regionName + " : " + cacheTable.getCache().toString());
//		return cacheTable.getCache();
//   }
//
//   public void putCache(String regionName, Object cache) {
//		CacheTable cacheTable = (CacheTable)table.get(regionName);
//      if (cacheTable != null) {
//         cacheTable.setCache(cache);
//      } else {
//         cacheTable = new CacheTable();
//         cacheTable.setCache(cache);
//         table.put(regionName,cacheTable);
//      }
//   }

   public Object getCache(String regionName) {
      CacheTable cacheTable = (CacheTable)cacheMap.get(regionName);
      if (cacheTable == null || cacheTable.getCache() == null) return null;
      WebPortalLogger.info("JCS Cache for " + regionName + " : " + cacheTable.getCache().toString());
      return cacheTable.getCache();
   }

   public void putCache(String regionName, Object cache) {
      CacheTable cacheTable = (CacheTable)cacheMap.get(regionName);
      if (cacheTable != null) {
         cacheTable.setCache(cache);
      } else {
         cacheTable = new CacheTable();
         cacheTable.setCache(cache);
         cacheMap.put(regionName,cacheTable);
      }
   }

//   public Object getCacheLoader(String regionName) {
//		CacheTable cacheTable = (CacheTable)table.get(regionName);
//      if (cacheTable == null || cacheTable.getCacheLoader() == null) return null;
//		return cacheTable.getCacheLoader();
//   }

   public Object getCacheLoader(String cacheLoaderClassName) {
      Object cacheLoader = cacheLoaderMap.get(cacheLoaderClassName);
      if (cacheLoader == null) {
         return null;
      }
      return cacheLoader;
   }

//   public void putCacheLoader(String regionName, Object cacheLoader) {
//      CacheTable cacheTable = (CacheTable)table.get(regionName);
//      if (cacheTable != null) {
//         cacheTable.setCacheLoader(cacheLoader);
//      } else {
//         cacheTable = new CacheTable();
//         cacheTable.setCacheLoader(cacheLoader);
//         table.put(regionName,cacheTable);
//      }
//   }

   public void putCacheLoader(Object cacheLoader) {
      String cacheLoaderClassName = ((ICacheLoader)cacheLoader).getClass().getName();
      Object cacheLoaderObject = cacheLoaderMap.get(cacheLoaderClassName);
      if (cacheLoaderObject != null) {
         // Cache Loader object already exists in the map.
         // So don't do anything.
      } else {
         cacheLoaderMap.put(cacheLoaderClassName, cacheLoader);
      }
   }

//   public boolean getScheduleExpiry(String regionName) {
//		CacheTable cacheTable = (CacheTable)table.get(regionName);
//		return cacheTable.getScheduleExpiry();
//   }
//
//   public void putScheduleExpiry(String regionName) {
//      CacheTable cacheTable = (CacheTable)table.get(regionName);
//      if (cacheTable != null) {
//         cacheTable.setScheduleExpiry(true);
//         scheduleExpiryItemCount++;
//      } else {
//         cacheTable = new CacheTable();
//         cacheTable.setScheduleExpiry(true);
//         scheduleExpiryItemCount++;
//         table.put(regionName,cacheTable);
//      }
//   }

   public boolean getScheduleExpiry(String regionName) {
      CacheTable cacheTable = (CacheTable)cacheMap.get(regionName);
      return cacheTable.getScheduleExpiry();
   }

   public void putScheduleExpiry(String regionName) {
      CacheTable cacheTable = (CacheTable)cacheMap.get(regionName);
      if (cacheTable != null) {
         cacheTable.setScheduleExpiry(true);
         scheduleExpiryItemCount++;
      } else {
         cacheTable = new CacheTable();
         cacheTable.setScheduleExpiry(true);
         scheduleExpiryItemCount++;
         cacheMap.put(regionName,cacheTable);
      }
   }

   public int getScheduleExpiryListCount() {
//      int itemCount=0;
//      Collection list = table.values();
//      Iterator it = list.iterator();
//      while (it.hasNext()) {
//         CacheTable item = (CacheTable)it.next();
//         if (item.getScheduleExpiry()) itemCount++;
//      }
//      return itemCount;
      return scheduleExpiryItemCount;
   }

   public List getScheduleExpiryList() {
      List list = new ArrayList();

      Collection values = cacheMap.values();
      Iterator it = values.iterator();
      while (it.hasNext()) {
         CacheTable item = (CacheTable)it.next();
         if (item.getScheduleExpiry()) {
            list.add(item);
         };
      }
      return list;
   }

   // INNER CLASS
   private class CacheTable {

   	String regionName;
   	Object cache;
   	Object cacheLoader;
   	boolean scheduleExpiry;

   	public CacheTable() {
   	}

      public CacheTable(String regionName, Object cache, Object cacheLoader,
   			boolean scheduleExpiry) {
   		this.regionName = regionName;
		   this.cache = cache;
		   this.cacheLoader = cacheLoader;
   		this.scheduleExpiry = scheduleExpiry;
   	}

      public Object getCache() {
         return this.cache;
      }

   	public void setCache(Object cache) {
			this.cache = cache;
		}

      public Object getCacheLoader() {
         return this.cacheLoader;
      }

   	public void setCacheLoader(Object cacheLoader) {
			this.cacheLoader = cacheLoader;
		}

      public boolean getScheduleExpiry() {
         return this.scheduleExpiry;
      }

   	public void setScheduleExpiry(boolean scheduleExpiry) {
			this.scheduleExpiry = scheduleExpiry;
		}

   }
}
// End of CacheList.java
