package common.caching.test;

import java.util.*;
import java.io.*;

import common.logging.WebPortalLogger;
import common.caching.CachingException;
import common.caching.*;


/**
 * Test Cache Loader class
 *
 * @version 1.0
 * @author Srini Penchikala
 * @since 10/03/2003
 */
public class TestCacheLoader implements ICacheLoader {

  /**
   * Attributes
   */
   private static long timeToSleep;
   private static String defaultCacheRegion = CacheRegions.DEFAULT_REGION;

   private static Random random = new Random(System.currentTimeMillis());
   private static ICacheManager manager;

   /**
    * Constructor
    */
   public TestCacheLoader() {
		manager = (BaseCacheManagerJCS)BaseCacheManagerJCS.getInstance();
   }

	/**
 	 * Data access logic implementation.
    * @param cacheKey Cache object
    * @param regionName Name of cache region
    * @return cached object
    * @throws CachingException
	 */
   public Object loadCacheObject(Object cacheKey, String regionName) throws CachingException {
      List list = null;

      if (cacheKey instanceof TestCacheKey) {
         // TestCacheKey
         TestCacheKey testCacheKey = new TestCacheKey();
         return testCacheKey.getData();
      } else if (cacheKey instanceof TestVehicleCacheKey) {
         // VechicleCacheTestKey
         TestVehicleCacheKey vehCacheTestKey = new TestVehicleCacheKey();
         return vehCacheTestKey.getData();
      } else {
         try {
            list = buildList();
         } catch (Exception e) {
            WebPortalLogger.info("Exception : " + e.toString());
            throw new CachingException(e);
         }
         return list;
      }
   }

   private LinkedList buildList() {
      LinkedList list = new LinkedList();

      for (int i = 0; i < 100; i++) {
         list.add(buildMap());
      }

      return list;
   }

   private HashMap buildMap() {
      HashMap map = new HashMap();

//      byte[] keyBytes = new byte[32];
//      byte[] valBytes = new byte[128];
      int keyBytes = 0;
      int valBytes = 0;

      for (int i = 0; i < 100; i++) {
//         random.nextBytes(keyBytes);
//         random.nextBytes(valBytes);
         keyBytes = random.nextInt();
         valBytes = random.nextInt();

         map.put((new Integer(keyBytes)).toString(), (new Integer(valBytes)).toString());
      }

      return map;
   }

   public Object getCache(String cacheKey) throws CachingException {
		return manager.getCacheObject(cacheKey, this);
	}

   public void removeCache(String cacheKey) throws CachingException {
		manager.removeCacheObject(cacheKey);
	}

}
// End of TestCacheLoader.java
