package common.logging;

import java.io.*;
import java.util.Properties;
//Import XML Classes
import javax.xml.parsers.*;
import javax.xml.parsers.ParserConfigurationException;

import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.w3c.dom.DOMException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;
import org.apache.log4j.Level;

public class WebPortalLogger {

//   static Logger logger = Logger.getLogger(WebPortalLogger.class);
   static Logger logger = Logger.getLogger("");
   static String propertiesFileName = "log4jconfig.properties";

   static {
      try {
         String log4JPropertyFile = "";
         Properties props = new Properties();
			InputStream in = WebPortalLogger.class.getResourceAsStream("/" + propertiesFileName);
			props.load(in);

         PropertyConfigurator.configure(props);
      } catch (FileNotFoundException fnfe) {
         System.out.println("WebPortalLogger.java - FileNotFoundException : " + fnfe.toString());
      } catch (IOException ioe) {
         System.out.println("WebPortalLogger.java - IOException : " + ioe.toString());
      } catch (Exception e) {
         System.out.println("WebPortalLogger.java - Exception : " + e.toString());
         e.printStackTrace();
      }
   }

   public static void debug (Object msg) {
      logger.debug(msg);
   }

   public static void info (Object msg) {
      logger.info(msg);
   }

   public static void warn (Object msg) {
      logger.warn(msg);
   }

   public static void error (Object msg) {
      logger.error(msg);
   }

   public static void fatal (Object msg) {
      logger.fatal(msg);
   }

   public static void printStackTrace(Throwable e) {
      StringWriter sw = new StringWriter();
      PrintWriter pw = new PrintWriter(sw);
      String beginStackTrace = "   ==== Begin Stack Trace for the Exception  ==== \n";
      String endStackTrace   = "\n =============  End of Stack Trace  =========== \n";
      String stackTrace = "";

      int level = logger.getEffectiveLevel().toInt();

      try {
         e.printStackTrace(pw);
         stackTrace = beginStackTrace + sw.toString() + endStackTrace;

         switch(level) {
            case Level.DEBUG_INT:
               debug(stackTrace); break;
            case Level.INFO_INT:
               info(stackTrace); break;
            case Level.ERROR_INT:
               error(stackTrace); break;
            case Level.FATAL_INT:
               fatal(stackTrace); break;
            case Level.ALL_INT:
               debug(stackTrace); break;
            default:
               debug(stackTrace); break;
         }
         sw.close();
         pw.close();
      } catch (Exception exc) {
         debug(exc.toString());
      }
   }

   public static void main(String args[]) {
//      DOMConfigurator.configure("log4jconfig.xml");

      logger.debug("Here is some DEBUG");
      logger.info("Here is some INFO");
      logger.warn("Here is some WARN");
      logger.error("Here is some ERROR");
      logger.fatal("Here is some FATAL");
   }

}
// End of WebPortalLogger.java
